using System;
using System.Collections;
using System.Diagnostics;
using EnvDTE;

namespace VSNUnit2k3
{
	public class PropertyUtils
	{
		private PropertyUtils() {}

		/// <summary>
		/// Return a Property as a string or null.
		/// An exception will be throw if property not a string.
		/// </summary>
		public static string GetPropertyString(Properties properties, string name)
		{
			Property property = GetProperty(properties, name);
			if(property == null) return null;
			return (string)property.Value;
		}

		/// <summary>
		/// Return a Property of one of the given names or null.
		/// </summary>
		public static Property GetProperty(Properties properties, string name)
		{
			try
			{
				return properties.Item(name);
			}
			catch(Exception)
			{
				return null;
			}
		}

		public static void DumpProperties(Properties properties)
		{
			foreach(Property property in properties)
			{
				Debug.WriteLine(property.Name + " = " + property.Value);
			}
		}

		public static IDictionary SaveProperties(Properties properties)
		{
			IDictionary dictionary = new Hashtable();
			foreach(Property property in properties)
			{
				dictionary[property.Name] = property.Value;
			}
			return dictionary;
		}

		public static void LoadProperties(Properties properties, IDictionary dictionary)
		{
			foreach(object key in dictionary.Keys)
			{
				properties.Item(key).Value = dictionary[key];
			}
		}
	}
}
