using System;

using NUnit.Framework;

using System.Collections;



namespace VSNUnit2k3

{



	#region UnitTests



	[TestFixture]

	public class TestTestSuites

	{



		private TestSuites _suites;



		[SetUp]

		public void Setup()

		{

			_suites = new TestSuites();

		}



		[TearDown]

		public void Teardown()

		{

			_suites.Dispose();

			_suites = null;

		}



		[Test]

		public void TestAddSuites()

		{

			Assert.IsTrue(_suites.Length == 0, "Initial length != 0");

			TestSuite suite = new TestSuite();

			suite.Name = "Test name 1";

			_suites.Add(suite);

			Assert.IsTrue(_suites.Length == 1, "Added 1, length != 1");

			TestSuite suite2 = new TestSuite();

			suite.Name = "Test name 2";

			_suites.Add(suite2);

			Assert.IsTrue(_suites.Length == 2, "Added 2, length != 2");

		}



		[Test]

		public void TestDeleteSuites()

		{

			Assert.IsTrue(_suites.Length == 0, "Initial length != 0");

			TestSuite suite1 = new TestSuite();

			suite1.Name = "Suite 1";

			TestSuite suite2 = new TestSuite();

			suite2.Name = "Suite 2";

			_suites.Add(suite1);

			_suites.Add(suite2);

			Assert.IsTrue(_suites.Length == 2, "Added 2, length != 2");

			_suites.Remove(1);

			Assert.IsTrue(_suites.Length == 1, "Removed 1, length != 1");

			_suites.Remove(1);

			Assert.IsTrue(_suites.Length == 1, "Removed at bad position, length != 1");

			_suites.Remove(suite2);

			Assert.IsTrue(_suites.Length == 1, "Removed bad member, length != 1");

			_suites.Remove(suite1);

			Assert.IsTrue(_suites.Length == 0, "Removed good member, length != 0");

		}



		[Test]

		public void TestIteratorSuites()

		{

			TestSuite s1 = new TestSuite();

			s1.Name = "Suite 1";

			TestSuite s2 = new TestSuite();

			s2.Name = "Suite 2";

			TestSuite s3 = new TestSuite();

			s3.Name = "Suite 3";



			_suites.Add(s1);

			_suites.Add(s2);

			_suites.Add(s3);



			Assert.IsTrue(_suites[0].Name == "Suite 1", "First suite != Suite 1");

			Assert.IsTrue(_suites[1].Name == "Suite 2", "Second suite != Suite 2");

			Assert.IsTrue(_suites[2].Name == "Suite 3", "Third suite != Suite 3");



			//			int i = 0;

			//			foreach(TestSuite suite in _suites)

			//			{

			//				i++;

			//			}

			//

			//			Assert.IsTrue(i == 3, "Total number of iterated suites != 3");

		}

		

	}



	[TestFixture]

	public class TestTestSuite

	{

		private TestSuite _suite;



		[SetUp]

		public void Setup()

		{

			_suite = new TestSuite();

		}



		[TearDown]

		public void TearDown()

		{

			_suite.Dispose();

			_suite = null;

		}



		[Test]

		public void TestSuiteProperties()

		{

			_suite.Name = "Test Suite";

			Assert.IsTrue(_suite.Name == "Test Suite", "Name not set properly");

			_suite.Name = "";

			Assert.IsTrue(_suite.Name == "Test Suite", "Blank name not filtered out");

		}



		[Test]

		public void TestSuiteCollections()

		{

			Assert.IsNotNull(_suite.Suites, "TestSuites collection not initialized in ctor");

			Assert.IsNotNull(_suite.Cases, "TestCases collection not initialized in ctor");

		}

	}



	[TestFixture]

	public class TestTestCases

	{



		private TestCases _cases;



		[SetUp]

		public void Setup()

		{

			_cases = new TestCases();

		}



		[TearDown]

		public void Teardown()

		{

			_cases.Dispose();

			_cases = null;

		}



		[Test]

		public void TestAddCases()

		{

			Assert.IsTrue(_cases.Length == 0, "Initial length != 0");

			TestCase case1 = new TestCase();

			case1.Name = "Test name 1";

			_cases.Add(case1);

			Assert.IsTrue(_cases.Length == 1, "Added 1, length != 1");

			TestCase case2 = new TestCase();

			case2.Name = "Test name 2";

			_cases.Add(case2);

			Assert.IsTrue(_cases.Length == 2, "Added 2, length != 2");

		}



		[Test]

		public void TestDeleteCases()

		{

			Assert.IsTrue(_cases.Length == 0, "Initial length != 0");

			TestCase case1 = new TestCase();

			case1.Name = "case 1";

			TestCase case2 = new TestCase();

			case2.Name = "case 2";

			_cases.Add(case1);

			_cases.Add(case2);

			Assert.IsTrue(_cases.Length == 2, "Added 2, length != 2");

			_cases.Remove(1);

			Assert.IsTrue(_cases.Length == 1, "Removed 1, length != 1");

			_cases.Remove(1);

			Assert.IsTrue(_cases.Length == 1, "Removed at bad position, length != 1");

			_cases.Remove(case2);

			Assert.IsTrue(_cases.Length == 1, "Removed bad member, length != 1");

			_cases.Remove(case1);

			Assert.IsTrue(_cases.Length == 0, "Removed good member, length != 0");

		}



		[Test]

		public void TestIteratorCases()

		{

			TestCase s1 = new TestCase();

			s1.Name = "case 1";

			TestCase s2 = new TestCase();

			s2.Name = "case 2";

			TestCase s3 = new TestCase();

			s3.Name = "case 3";



			_cases.Add(s1);

			_cases.Add(s2);

			_cases.Add(s3);



			Assert.IsTrue(_cases[0].Name == "case 1", "First case != case 1");

			Assert.IsTrue(_cases[1].Name == "case 2", "Second case != case 2");

			Assert.IsTrue(_cases[2].Name == "case 3", "Third case != case 3");



			//			int i = 0;

			//			foreach(Testcase case in _cases)

			//			{

			//				i++;

			//			}

			//

			//			Assert.IsTrue(i == 3, "Total number of iterated cases != 3");

		}

		

	}



	[TestFixture]

	public class TestTestCase

	{

		private TestCase _case;



		[SetUp]

		public void Setup()

		{

			_case = new TestCase();

		}



		[TearDown]

		public void Teardown()

		{

			_case = null;

		}



		[Test]

		public void TestProperties()

		{		

			_case.Name = "Case Name";

			_case.Status = TestCaseStatus.Failed;

			_case.Message = "Case Message";

			_case.LineNumber = 2;

			_case.FileName = "thefile.cs";

			_case.StackTrace = "stacktrace";





			Assert.IsTrue(_case.Name == "Case Name");

			Assert.IsTrue(_case.Status == TestCaseStatus.Failed);

			Assert.IsTrue(_case.Message == "Case Message");

			Assert.IsTrue(_case.LineNumber == 2);

			Assert.IsTrue(_case.FileName == "thefile.cs");

			Assert.IsTrue(_case.StackTrace == "stacktrace");

		}	



		[Test]

		public void TestHasSource()

		{

			Assert.IsFalse(_case.HasSource, "default condition should be ! HasSource");

			_case.LineNumber = 12;

			Assert.IsFalse(_case.HasSource, "just a line number should be ! HasSource");

			_case.LineNumber = -1;

			_case.FileName = "asdfasdf";

			Assert.IsFalse(_case.HasSource, "just a filename should be ! HasSource");

			_case.LineNumber = 12;

			Assert.IsTrue(_case.HasSource, "filename plus line number should be HasSource");

		}

	}

	#endregion





	public class TestSuites : IDisposable

	{

		private ArrayList _suites;



		public TestSuites()

		{

			_suites = new ArrayList();

		}



		public void Dispose()

		{

			_suites.Clear();

		}



		public int Length

		{

			get

			{

				return _suites.Count;

			}

		}



		public void Add(TestSuite suite)

		{

			_suites.Add(suite);

		}



		public void Remove(int index)

		{

			try

			{

				_suites.RemoveAt(index);

			}

			catch

			{



			}

		}



		public void Remove(TestSuite suite)

		{

			try

			{

				_suites.Remove(suite);

			}

			catch

			{

			}

		}



		public TestSuite this[int index]

		{

			get

			{

				return (TestSuite)_suites[index];

			}

			set

			{

				_suites[index] = value;

			}

		}

	}



	public class TestCases : IDisposable

	{

		private ArrayList _cases;



		public TestCases()

		{

			_cases = new ArrayList();

		}



		public void Dispose()

		{

			_cases.Clear();

		}



		public int Length

		{

			get

			{

				return _cases.Count;

			}

		}



		public void Add(TestCase _case)

		{

			_cases.Add(_case);

		}



		public void Remove(int index)

		{

			try

			{

				_cases.RemoveAt(index);

			}

			catch

			{



			}

		}



		public void Remove(TestCase _case)

		{

			try

			{

				_cases.Remove(_case);

			}

			catch

			{

			}

		}



		public TestCase this[int index]

		{

			get

			{

				return (TestCase)_cases[index];

			}

			set

			{

				_cases[index] = value;

			}

		}

	}



	public class TestSuite : IDisposable

	{

		private string _name;

		private TestSuites _suites;

		private TestCases _cases;



		public TestSuite()

		{

			_suites = new TestSuites();

			_cases = new TestCases();

		}



		public void Dispose()

		{

			_suites.Dispose();

			_cases.Dispose();

		}



		public string Name

		{

			get

			{

				return _name;

			}

			set

			{

				if(value != "")

					_name = value;

			}

		}



		public TestSuites Suites

		{

			get

			{

				return _suites;

			}

		}



		public TestCases Cases

		{

			get

			{

				return _cases;

			}

		}

		



	}



	public enum TestCaseStatus

	{

		Failed = 0,

		Succeeded = 1,

		NotRun = 2

	}



	public class TestCase

	{

		private string _name;

		private string _message;

		private string _filename = "";

		private TestCaseStatus _status;

		private int _linenumber = -1;

		private string _stacktrace;



		public string Name

		{

			get

			{

				return _name;

			}

			set

			{

				if(value != "")

					_name = value;

			}

		}



		public string Message

		{

			get

			{

				return _message;

			}

			set

			{	

				if(value != "")

					_message = value;

			}

		}



		public string FileName

		{

			get

			{

				return _filename;

			}

			set

			{

				_filename = value;

			}

		}



		public TestCaseStatus Status

		{

			get

			{

				return _status;

			}

			set

			{

				_status = value;

			}

		}



		public int LineNumber

		{

			get

			{

				return _linenumber;

			}

			set

			{   

				_linenumber = value;

			}

		}



		public string StackTrace

		{

			get

			{

				return _stacktrace;

			}

			set

			{

				if(value != "")

					_stacktrace = value;

			}

		}



		public bool HasSource

		{

			get

			{

				if(_linenumber < 0 || _filename == "")

					return false;

				return true;

			}

		}

	}	

}

