using System;

using System.Collections;

using System.ComponentModel;

using System.Drawing;

using System.Data;

using System.Windows.Forms;

using System.Xml;

using System.Text;

using EnvDTE;

using VSLangProj;

using NUnit.Framework;



namespace VSNUnit2k3

{



	#region Enums

	/// <summary>

	/// Indexes into the ImageList control associated with the results tree view.

	/// </summary>

	public enum vsnTreeIcons

	{

		ErrorImg = 0,

		SuccessImg = 1,

		MessageImg = 2,

		None = 3

	}

	#endregion





	/// <summary>

	/// UI for displaying test results in a heirarchical tree, and allowing users to quickly jump to failed

	/// tests from results.

	/// </summary>

	public class vsnunitctl : System.Windows.Forms.UserControl

	{



		#region Declarations



		public		CNUnitWrapper		cnu;

		

		internal	System.Windows.Forms.TreeView tvResults;

		

		private ArrayList failures;

		private		_DTE				_app;

		private System.Windows.Forms.ImageList ilTree;

		private System.ComponentModel.IContainer components;

		private System.Windows.Forms.OpenFileDialog openFileDialog1;

		private System.Windows.Forms.Panel panel1;

		private System.Windows.Forms.Splitter splitter1;

		private System.Windows.Forms.SaveFileDialog saveFileDialog1;

		private System.Windows.Forms.ToolBar toolBar1;

		private System.Windows.Forms.ToolBarButton tbbTest;

		private System.Windows.Forms.ToolBarButton tbbChoose;

		private System.Windows.Forms.ToolBarButton tbbSave;

		private System.Windows.Forms.ToolBarButton toolBarButton1;

		private System.Windows.Forms.ImageList ilToolBar;

		private System.Windows.Forms.ToolBarButton toolBarButton2;

		private System.Windows.Forms.ToolBarButton tbbNextFail;

		private System.Windows.Forms.ToolBarButton tbbPrevFail;

		private System.Windows.Forms.ToolBarButton toolBarButton3;

		private System.Windows.Forms.ToolBarButton tbbAbout;

		private System.Windows.Forms.ToolBarButton tbbOnBuild;

		

		#endregion



		#region CTors/~CTors



		public vsnunitctl()

		{

			// This call is required by the Windows.Forms Form Designer.

			InitializeComponent();



			cnu = new CNUnitWrapper();

			failures = new ArrayList();

		}



		/// <summary> 

		/// Clean up any resources being used.

		/// </summary>

		protected override void Dispose( bool disposing )

		{

			if( disposing )

			{

				if(components != null)

				{

					components.Dispose();

				}

			}

			base.Dispose( disposing );

		}



		#endregion



		#region Properties



		/// <summary>

		/// Capture the VS.NET extensibility object

		/// </summary>

		public _DTE App

		{

			get

			{

				return _app;

			}

			set

			{

				_app = value;

				_app.Events.BuildEvents.OnBuildDone += new _dispBuildEvents_OnBuildDoneEventHandler(this.OnBuildDone);

				_app.Events.SolutionEvents.Opened += new _dispSolutionEvents_OpenedEventHandler(this.OnProjectOpened);

				_app.Events.SolutionEvents.AfterClosing += new _dispSolutionEvents_AfterClosingEventHandler(OnAfterProjectClosd);

				

			//	CheckForNUnit();

			}

		}

		#endregion



		#region Event Handlers



		private void toolBar1_ButtonClick(object sender, System.Windows.Forms.ToolBarButtonClickEventArgs e)

		{

			switch(toolBar1.Buttons.IndexOf(e.Button))

			{

				case 0: //Test

					_app.Solution.SolutionBuild.Build(true);

					

					StartTest();

					break;



				case 1: //Choose NUnit

					if(openFileDialog1.ShowDialog()==DialogResult.OK)

					{

						cnu.NUnitExec = openFileDialog1.FileName;

						cnu.CusPath = true;

					}

					break;



				case 2: //Save results

					string temp = GetProjectName();
					if(temp!= null){
						saveFileDialog1.FileName = temp + ".xml";
						
					}
					saveFileDialog1.Filter = "xml files (*.xml)|*.xml|All files (*.*)|*.*";
					if(saveFileDialog1.ShowDialog() == DialogResult.OK)

					{

						cnu.SaveResults(saveFileDialog1.FileName);

					}

					break;

				case 4: //Test On Build

					cnu.TestOnBuild = e.Button.Pushed;

					break;

				case 6: //Next fail

					NextFail();

					break;

				case 7: //Previous fail

					PrevFail();

					break;

				case 9: //About

					MessageBox.Show("VSNUnit is an open source add-in for Visual Studio.NET, \n originally authored by Relevance, LLC (www.relevancellc.com). \n For more information, please visit www.relevancellc.com/vsnunit.", "About VSNUnit", MessageBoxButtons.OK, MessageBoxIcon.None);

					break;

			}

		}




		private void tvResults_AfterSelect(object sender, System.Windows.Forms.TreeViewEventArgs e)

		{

			tvResults.SelectedNode.SelectedImageIndex = tvResults.SelectedNode.ImageIndex;

		}



		/// <summary>

		/// Double clicking a test case in the treeview should result in:

		/// 1) if test failed, open source code and highlight failed test

		/// 2) if test succeeded or not run, do nothing

		/// </summary>

		/// <param name="sender">treeview that raised the event</param>

		/// <param name="e">event args</param>

		private void tvResults_DoubleClick(object sender, System.EventArgs e)

		{

			TreeNode node = tvResults.SelectedNode;



			//if the node represents a testcase or testsuite

			if(node.Tag != null)

			{

				if(node.Tag is TestSuite)

				{

					TestSuite suite = (TestSuite)node.Tag;

				}

				else

				{

					TestCase mycase = (TestCase)node.Tag;

					ShowSource(mycase);

				}

			}

			else

			{

				TestCase mycase = (TestCase)node.Parent.Tag;

				if(mycase != null)

					ShowSource(mycase);

			}



			//TODO: Total hack. Need a way to turn off auto-expand/collapse on double click.  The CancelEventArgs has no way of

			//knowing how or why the expand/collapse event was triggered.  There is no programmatic way to distinguish between a user clicking

			//on the +/- or double clicking on a node.  I am using the boolean node.checked value (which is unused otherwise as the treeview

			//is not a checked treeview) to pass a value to the CancelEventHandler.  It checks to see if this value is false, which only 

			//happens during a double click event. If the value is false, it cancels the event thus preventing the auto-expand/collapse, then resets

			//the value to true.

			node.Checked = false;



		}



		



		/// <summary>

		/// Event handler for when a project is built.  If the user has elected to run tests automatically upon a build,

		/// this will fire the test handler.

		/// </summary>

		/// <param name="scope"></param>

		/// <param name="action"></param>

		public void OnBuildDone(vsBuildScope scope, vsBuildAction action)

		{

			

			if(cnu.TestOnBuild)

				StartTest();

		}



		/// <summary>

		/// When a new project is opened, check for the current version of NUnit referenced.

		/// </summary>

		public void OnProjectOpened()

		{

	//		MessageBox.Show("Opened");

	//		CheckForNUnit();
			
	//		_app.ExecuteCommand("VSNUnit2k3.Connect.VSNUnit2k3","");

	//		_app.Windows.Item("VSNUnit2k3").Visible = true;

			

			tvResults.Nodes.Clear();	

		}



		private void tvResults_BeforeExpand(object sender, System.Windows.Forms.TreeViewCancelEventArgs e)

		{

			TreeNode node = ((TreeView)sender).SelectedNode;

			if(node != null)

			{

				if(!node.Checked)

				{

					node.Checked = true;

					e.Cancel = true;

				}

			}

		}



		private void tvResults_BeforeCollapse(object sender, System.Windows.Forms.TreeViewCancelEventArgs e)

		{

			TreeNode node = ((TreeView)sender).SelectedNode;

			if(node != null)

			{

				if(!node.Checked)

				{

					node.Checked = true;

					e.Cancel = true;

				}

			}

		}



		private void vsnunitctl_Load(object sender, System.EventArgs e)

		{

			//CheckForNUnit();

		}



		#endregion



		#region Implementation





		/// <summary>

		/// Given a library, execute the testrunner.

		/// </summary>

		/// <param name="rootpath">The full path to the library to test.</param>

		/// <returns>The XML document representing the results of the test.</returns>

		internal XmlDocument TestAndBuild(string rootpath)

		{



			failures.Clear();

			



			XmlDocument results;

			

			//Rootpath assumes a .dll extension

			if(System.IO.File.Exists(rootpath))

			{

				results = cnu.Test(rootpath);
				

				return results;

			}

			//if assembly not .dll, try .exe

			else if(System.IO.File.Exists(rootpath.Replace(".dll", ".exe")))

			{

				results = cnu.Test(rootpath.Replace(".dll", ".exe"));

				return results;

			}

			else

			{

			
				return null;

			}			

		}



		/// <summary>

		/// Given a loaded project, check to see if the project references a version of NUnit.  Use the base path of the reference

		/// to auto-configure the appropriate nunit-console.exe version to run.

		/// </summary>

		private void CheckForNUnit()

		{

			//Get current project

			System.Array arr = (System.Array)_app.ActiveSolutionProjects;

			Project p = (Project)arr.GetValue(0);

		//	Project p = GetTestProject();

			

			//Get VSProject interface reference to current project

			VSProject vsp = (VSProject)p.Object;

			foreach(Reference rf in vsp.References)

			{
			//	MessageBox.Show(rf.Name);

				if(rf.Name == "nunit.framework")

				{
			//		MessageBox.Show(rf.Path);

					
					string consolepath = rf.Path.Replace( @"\\",@"\");

					consolepath = consolepath.Replace("nunit.framework.dll", "nunit-console.exe");

					cnu.NUnitExec = consolepath;

					break;

				}

			}

		}



		/// <summary>

		/// Navigate to next failure node

		/// </summary>

		private void NextFail()

		{

			TreeNode current = getClosestCaseNode();

			TreeNode next = null;

			if(failures.Contains(current))

			{

				int curpos = failures.IndexOf(current);

				if(curpos < failures.Count - 1)

				{

					next = (TreeNode)failures[curpos + 1];

				}

				else

				{

					next = (TreeNode)failures[0];

				}

			}

			else

			{

				next = (TreeNode)failures[0];

			}

			

			TestCase tc = next.Tag as TestCase;

			tvResults.SelectedNode = next;

			next.EnsureVisible();

			ShowSource(tc);

			tvResults.Select();

			tvResults.Focus();

		}



		/// <summary>

		/// Navigate to previous failure node

		/// </summary>

		private void PrevFail()

		{

			TreeNode current = getClosestCaseNode();

			TreeNode next = null;

			if(failures.Contains(current))

			{

				int curpos = failures.IndexOf(current);

				if(curpos > 0)

				{

					next = (TreeNode)failures[curpos - 1];

				}

				else

				{

					next = (TreeNode)failures[failures.Count - 1];

				}

			}

			else

			{

				next = (TreeNode)failures[0];

			}

			

			TestCase tc = next.Tag as TestCase;

			tvResults.SelectedNode = next;

			next.EnsureVisible();

			ShowSource(tc);

			tvResults.Select();

			tvResults.Focus();

		}



		/// <summary>

		/// Returns closest case node to current selection. If selection is beneath case, returns parent case. If it is a suite,

		/// return first case in the suite.

		/// </summary>

		/// <returns></returns>

		private TreeNode getClosestCaseNode()

		{

			TreeNode selected = tvResults.SelectedNode;

			if(selected == null)  selected = tvResults.Nodes[0];



			TreeNode casenode = GetCase(selected);

			return casenode;

		}



		/// <summary>

		/// Tests a node for being a case; recurses down (or up) the tree.

		/// </summary>

		/// <param name="current"></param>

		/// <returns></returns>

		private TreeNode GetCase(TreeNode current)

		{

			if(current.Tag is TestCase)

				return current;



			if(current.Nodes.Count > 0)

			{

				return GetCase(current.Nodes[0]);

			}

			else

			{

				return current.Parent;

			}

		}


		private void AdditionInfo(TreeNode tree,ArrayList info){
			TreeNode treeConsole;
			TreeNode tn;
			treeConsole = new TreeNode("System Console Write lines:");
			treeConsole.ImageIndex = (int)vsnTreeIcons.SuccessImg;
			System.Collections.IEnumerator i = info.GetEnumerator();
			while(i.MoveNext()){
				tn = new TreeNode((string)i.Current);
				tn.ImageIndex = (int)vsnTreeIcons.SuccessImg;
				treeConsole.Nodes.Add(tn);
			}
			tree.Nodes.Add(treeConsole);
		}


		/// <summary>

		/// Fill in the treeview for graphically displaying the results.

		/// </summary>

		/// <param name="source">The output from TestAndBuild</param>

		internal void FillTreeview(XmlDocument source)

		{
			
			

			if(source != null) 
			{
			

				tvResults.Nodes.Clear();

				XmlNode testroot = source.DocumentElement;

				TestSuite suite = new TestSuite();

				suite.Name = testroot.Attributes.GetNamedItem("name").Value;

				TreeNode treeroot = new TreeNode(suite.Name);

				AdditionInfo(treeroot,cnu.beginShow);

				treeroot.Tag = suite;

				RecurseResults(testroot, treeroot);

			//	AdditionInfo(treeroot,cnu.endShow);

				tvResults.Nodes.Add(treeroot);
			}

			

			tvResults.ExpandAll();

		}



		/// <summary>

		/// Clears the tree of results, and puts a "processing" note as the only node.

		/// </summary>

		internal void ShowTreeProcessing()

		{

			tvResults.Nodes.Clear();

			tvResults.Nodes.Add("Testing...");

		}



		/// <summary>

		/// Recursively create treenodes from the XML results.

		/// </summary>

		/// <param name="inxmlnode">The current XML node to parse.</param>

		/// <param name="intreenode">The current tree node to fill in.</param>

		private void RecurseResults(XmlNode inxmlnode, TreeNode intreenode)

		{

			if(inxmlnode == null) return;

			intreenode.ImageIndex = (int)vsnTreeIcons.SuccessImg;

			if(inxmlnode.ChildNodes == null) return;



			foreach(XmlNode node in inxmlnode.ChildNodes)

			{

				if(node.Name == "test-suite")

				{

					TreeNode suitenode = new TreeNode(node.Attributes.GetNamedItem("name").Value + "  ( " + node.Attributes.GetNamedItem("time").Value + " )");

					

					//Concurrently build object tree that matches treeview

					TestSuite parentsuite = (TestSuite)intreenode.Tag;

					TestSuite childsuite = new TestSuite();

					childsuite.Name = node.Attributes.GetNamedItem("name").Value;

					parentsuite.Suites.Add(childsuite);

					suitenode.Tag = childsuite;



					RecurseResults(node.SelectSingleNode("results"), suitenode);

					

					if(suitenode.ImageIndex == (int)vsnTreeIcons.ErrorImg) intreenode.ImageIndex = (int)vsnTreeIcons.ErrorImg;

					intreenode.Nodes.Add(suitenode);

				}

				else if(node.Name == "test-case")

				{

					TreeNode casenode = new TreeNode();

					

					//Concurrently build object tree that matches treeview

					TestSuite parentsuite = (TestSuite)intreenode.Tag;

					TestCase childcase = new TestCase();

					casenode.Tag = childcase;

					if(node.Attributes.GetNamedItem("executed").Value == "False"){
						childcase.Name = node.Attributes.GetNamedItem("name").Value;
					}
					else{
						childcase.Name = node.Attributes.GetNamedItem("name").Value +  "  ( " + node.Attributes.GetNamedItem("time").Value + " )" ;
					}
					

					casenode.Text = childcase.Name;



					if(node.Attributes.GetNamedItem("executed").Value == "False")

					{
						

						childcase.Status = TestCaseStatus.NotRun;

						RecurseResults(node.SelectSingleNode("reason"), casenode);

						casenode.ImageIndex = (int)vsnTreeIcons.MessageImg;

					}

					else

					{
				//		childcase.Name = node.Attributes.GetNamedItem("name").Value +  "  ( " + node.Attributes.GetNamedItem("time").Value + " )" ;

						if(node.Attributes.GetNamedItem("success").Value == "True")

						{

							childcase.Status = TestCaseStatus.Succeeded;

							casenode.ImageIndex = (int)vsnTreeIcons.SuccessImg;

						}

						else

						{

							childcase.Status = TestCaseStatus.Failed;

							RecurseResults(node.SelectSingleNode("failure"), casenode);

							casenode.ImageIndex = (int)vsnTreeIcons.ErrorImg;

							intreenode.ImageIndex = (int)vsnTreeIcons.ErrorImg;	

							AddToFailures(casenode);

						}


					}
					intreenode.Nodes.Add(casenode);
					


				}

				else if(node.Name == "message")

				{

					TreeNode messagenode = new TreeNode(node.InnerText.Replace("![CDATA[", "").Replace("]]", ""));

					TestCase parentcase = (TestCase)intreenode.Tag;

					parentcase.Message = messagenode.Text;

					messagenode.ImageIndex = (int)vsnTreeIcons.None;

					intreenode.Nodes.Add(messagenode);

				}	

				else if(node.Name == "stack-trace")

				{

				//	TreeNode stacknode = new TreeNode(node.InnerText.Replace("![CDATA[", "").Replace("]]", ""));

					string temp = node.InnerText.Replace("![CDATA[", "").Replace("]]", "");
					
					temp = temp.Remove(0,temp.LastIndexOf("   at "));

					TreeNode stacknode = new TreeNode(temp);

					TestCase parentcase = (TestCase)intreenode.Tag;

					parentcase.StackTrace = stacknode.Text;

					ParseSource(stacknode.Text, ref parentcase);

					stacknode.ImageIndex = (int)vsnTreeIcons.None;

					intreenode.Nodes.Add(stacknode);

				}

			}

		}



		/// <summary>

		/// Appends tree node to list of failure nodes for easy next/prev navigation.

		/// </summary>

		/// <param name="failure">The tree node that represents a failed test.</param>

		private void AddToFailures(TreeNode failure)

		{

			failures.Add(failure);

		}



		/// <summary>

		/// Given the stacktrace message of a failed test, pull out the filename and line number where the test failed,

		/// and update the TestCase instance.

		/// </summary>

		/// <param name="sourceline">The output from the stack trace.</param>

		/// <param name="tcase">The TestCase to modify with the new values.</param>

		internal void ParseSource(string sourceline, ref TestCase tcase)

		{

			int indexOfIn = sourceline.IndexOf(" in ");

			if(indexOfIn < 0)

				return;



			//get the filename (fullpath)

			string file = sourceline.Substring(sourceline.IndexOf(" in ") + 4);

			file = file.Substring(0, file.IndexOf(":line"));

			tcase.FileName = file;



			//get the line number

			string linenumber = sourceline.Substring(sourceline.LastIndexOf(":"));

			linenumber = linenumber.Substring(linenumber.IndexOf(" "));

			int ln = Convert.ToInt32(linenumber);

			tcase.LineNumber = ln;

		}



		/// <summary>

		/// Check to see if project is VB or "other". 

		/// </summary>

		/// <param name="project">The project to verify</param>

		/// <returns>True if VB.NET, false if anything else</returns>

		private bool IsVBProject(Project project)

		{

			if(project.FileName.IndexOf(".vbproj") > 0)

				return true;

			return false;

		}



		/// <summary>

		/// Using the application object, find the library, run the tests, and display them in the treeview.

		/// </summary>

		private void StartTest()

		{

			try

			{

				if(!cnu.CusPath)
				{
					CheckForNUnit();
				}

				//show the "processing" note

				ShowTreeProcessing();



				//get the current project

				System.Array arr = (System.Array)_app.ActiveSolutionProjects;

				Project p = (Project)arr.GetValue(0);

		//		Project p = GetTestProject();

				

				//get the root path to the debug (or release) directory where the library will reside

				string rootpath = getOutputFilePath(p);

				

				//Start the tests

				XmlDocument results = TestAndBuild(rootpath);



				//If results are returned, display them to user

				if(results!=null)

					FillTreeview(results);

			}

			catch(Exception ex)

			{

				if(ex.Message.StartsWith("There is invalid data at the root level"))

				{

					MessageBox.Show("Sorry, still implementing NUnit v2.0.", "Version not supported");

				}

				else if(ex.Message.StartsWith("Object reference not set"))

				{

					MessageBox.Show("You must first add a reference to nunit.framework.dll \n to your project, or choose a version of nunit-console.exe to use.", "VSNUnit error", MessageBoxButtons.OK, MessageBoxIcon.Warning);

				}

				else

				{

					MessageBox.Show(ex.Message.GetType().FullName + "\n" + ex.Message + "\n" + ex.StackTrace, "Error(s)...");

				}

			}

			

		}

		private Project GetTestProject(){
			try
			{
			
				System.Array arr = (System.Array)_app.ActiveSolutionProjects;

				Project p = (Project)arr.GetValue(0);

				int counts = arr.Length;

				for(int i=0;i<counts;i++){
					p = (Project)arr.GetValue(i);
					if(p.Name.ToLower().IndexOf("test") != -1){
						break;
					}
					if((i+1) == counts){
						p = (Project)arr.GetValue(0);
					}
				}
				
				return p;
			}
			catch (Exception ex) 
			{
				MessageBox.Show(ex.Message.GetType().FullName + "\n" + ex.Message + "\n" + ex.StackTrace, "Error(s)...");
				return null;
			}
		}

		private string GetProjectName()
		{
			try
			{
			
				System.Array arr = (System.Array)_app.ActiveSolutionProjects;
		
				Project p = (Project)arr.GetValue(0);
				
		//		Project p = GetTestProject();

				return p.Properties.Item("OutputFileName").Value.ToString();
			}
			catch (Exception ex) 
			{
				MessageBox.Show(ex.Message.GetType().FullName + "\n" + ex.Message + "\n" + ex.StackTrace, "Error(s)...");
				return null;
			}
		}




		private bool GotoSource(ProjectItem pi,TestCase tc){
			//if the document's path matches the TestCase filename

			if(pi.Document.FullName.ToLower() == tc.FileName.ToLower())

			{

				//show the document, goto the line and highlight it

		//		MessageBox.Show(tc.FileName + "       " + tc.LineNumber);

				pi.Document.Activate();
							
		//		MessageBox.Show("pi.active");

				_app.ExecuteCommand("Edit.GoTo", tc.LineNumber.ToString());

		//		MessageBox.Show("Edit.GoTo");

				((TextSelection)_app.ActiveDocument.Selection).SelectLine();

				return true;
			}
			return false;
		}

		private bool ShowSourceCode(ProjectItem pi,TestCase tc){
			if(pi.ProjectItems.Count != 0){
				foreach(ProjectItem npi in pi.ProjectItems){
					if(ShowSourceCode(npi,tc)){
						break;
					}
				}
			}
			else{
				if(pi.Document != null){
					return GotoSource(pi,tc);
				}
			}
			return false;
		}


		/// <summary>

		/// Given a failed TestCase, display the source file and highlight the line where the test failed.

		/// </summary>

		/// <param name="tc"></param>

		internal void ShowSource(TestCase tc)

		{

			//if the project is in Release mode, the source will not be available

			if(!tc.HasSource)

			{

				MessageBox.Show("Project is in Release mode. Source of failure not available.", "Source Unavailable",MessageBoxButtons.OK, MessageBoxIcon.Asterisk);

				return;

			}





			//Only if the test failed

			if(tc.Status == TestCaseStatus.Failed)

			{

				//Get the current project

				System.Array arr = (System.Array)_app.ActiveSolutionProjects;

				Project p = (Project)arr.GetValue(0);

		//		Project p = GetTestProject();

				bool found = false;


				foreach(ProjectItem pi in p.ProjectItems){
					if((found = ShowSourceCode(pi,tc)) == true){
						break;
					}
				}


				
		//		MessageBox.Show(tc.FileName + "       " + tc.LineNumber);


				//See if the code file is already open (either focused or not)

		/*		foreach(ProjectItem pi in p.ProjectItems)
		
				{

					if(pi.ProjectItems.Count != 0){
						foreach(ProjectItem npi in pi.ProjectItems){
							if(GotoSource(npi,tc)){
								break;
							}
						}
					}
					//If the item is a document item

					if(pi.Document != null)

					{



					}

				}

		*/

				//if the document was not already open

				if(!found)

				{

					//open the document, activate it, goto the line and highlight it

					Window win = _app.ItemOperations.OpenFile(tc.FileName, Constants.vsViewKindCode);

					win.Activate();

					_app.ExecuteCommand("Edit.GoTo", tc.LineNumber.ToString());

					((TextSelection)_app.ActiveDocument.Selection).SelectLine();

				}

			}

		}



		/// <summary>

		/// Given a Project, determine the full path to its output file to feed to NUnit.

		/// </summary>

		/// <param name="p">The active project.</param>

		/// <returns>The full absolute path to the output file.</returns>

		private string getOutputFilePath(Project p)

		{

			StringBuilder output = new StringBuilder();

			output.Append(p.Properties.Item("FullPath").Value.ToString());

			output.Append(p.ConfigurationManager.ActiveConfiguration.Properties.Item("OutputPath").Value.ToString());

			output.Append(p.Properties.Item("OutputFileName").Value.ToString());

			return output.ToString();

		}



		#endregion		



		#region Component Designer generated code

		/// <summary> 

		/// Required method for Designer support - do not modify 

		/// the contents of this method with the code editor.

		/// </summary>

		private void InitializeComponent()

		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(vsnunitctl));
			this.ilTree = new System.Windows.Forms.ImageList(this.components);
			this.openFileDialog1 = new System.Windows.Forms.OpenFileDialog();
			this.panel1 = new System.Windows.Forms.Panel();
			this.toolBar1 = new System.Windows.Forms.ToolBar();
			this.tbbTest = new System.Windows.Forms.ToolBarButton();
			this.tbbChoose = new System.Windows.Forms.ToolBarButton();
			this.tbbSave = new System.Windows.Forms.ToolBarButton();
			this.toolBarButton1 = new System.Windows.Forms.ToolBarButton();
			this.tbbOnBuild = new System.Windows.Forms.ToolBarButton();
			this.toolBarButton2 = new System.Windows.Forms.ToolBarButton();
			this.tbbNextFail = new System.Windows.Forms.ToolBarButton();
			this.tbbPrevFail = new System.Windows.Forms.ToolBarButton();
			this.toolBarButton3 = new System.Windows.Forms.ToolBarButton();
			this.tbbAbout = new System.Windows.Forms.ToolBarButton();
			this.ilToolBar = new System.Windows.Forms.ImageList(this.components);
			this.splitter1 = new System.Windows.Forms.Splitter();
			this.tvResults = new System.Windows.Forms.TreeView();
			this.saveFileDialog1 = new System.Windows.Forms.SaveFileDialog();
			this.panel1.SuspendLayout();
			this.SuspendLayout();
			// 
			// ilTree
			// 
			this.ilTree.ImageSize = new System.Drawing.Size(16, 16);
			this.ilTree.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("ilTree.ImageStream")));
			this.ilTree.TransparentColor = System.Drawing.Color.Transparent;
			// 
			// openFileDialog1
			// 
			this.openFileDialog1.FileOk += new System.ComponentModel.CancelEventHandler(this.openFileDialog1_FileOk);
			// 
			// panel1
			// 
			this.panel1.Controls.Add(this.toolBar1);
			this.panel1.Dock = System.Windows.Forms.DockStyle.Top;
			this.panel1.Location = new System.Drawing.Point(0, 0);
			this.panel1.Name = "panel1";
			this.panel1.Size = new System.Drawing.Size(528, 24);
			this.panel1.TabIndex = 3;
			// 
			// toolBar1
			// 
			this.toolBar1.Buttons.AddRange(new System.Windows.Forms.ToolBarButton[] {
																						this.tbbTest,
																						this.tbbChoose,
																						this.tbbSave,
																						this.toolBarButton1,
																						this.tbbOnBuild,
																						this.toolBarButton2,
																						this.tbbNextFail,
																						this.tbbPrevFail,
																						this.toolBarButton3,
																						this.tbbAbout});
			this.toolBar1.DropDownArrows = true;
			this.toolBar1.ImageList = this.ilToolBar;
			this.toolBar1.Location = new System.Drawing.Point(0, 0);
			this.toolBar1.Name = "toolBar1";
			this.toolBar1.ShowToolTips = true;
			this.toolBar1.Size = new System.Drawing.Size(528, 28);
			this.toolBar1.TabIndex = 4;
			this.toolBar1.ButtonClick += new System.Windows.Forms.ToolBarButtonClickEventHandler(this.toolBar1_ButtonClick);
			// 
			// tbbTest
			// 
			this.tbbTest.ImageIndex = 2;
			this.tbbTest.ToolTipText = "Run Tests";
			// 
			// tbbChoose
			// 
			this.tbbChoose.ImageIndex = 1;
			this.tbbChoose.ToolTipText = "Choose NUnit";
			// 
			// tbbSave
			// 
			this.tbbSave.ImageIndex = 5;
			this.tbbSave.ToolTipText = "Save Results";
			// 
			// toolBarButton1
			// 
			this.toolBarButton1.Style = System.Windows.Forms.ToolBarButtonStyle.Separator;
			// 
			// tbbOnBuild
			// 
			this.tbbOnBuild.ImageIndex = 6;
			this.tbbOnBuild.Style = System.Windows.Forms.ToolBarButtonStyle.ToggleButton;
			this.tbbOnBuild.ToolTipText = "Test On Build";
			// 
			// toolBarButton2
			// 
			this.toolBarButton2.Style = System.Windows.Forms.ToolBarButtonStyle.Separator;
			// 
			// tbbNextFail
			// 
			this.tbbNextFail.ImageIndex = 3;
			this.tbbNextFail.ToolTipText = "Next Failed Test";
			// 
			// tbbPrevFail
			// 
			this.tbbPrevFail.ImageIndex = 4;
			this.tbbPrevFail.ToolTipText = "Previous Failed Test";
			// 
			// toolBarButton3
			// 
			this.toolBarButton3.Style = System.Windows.Forms.ToolBarButtonStyle.Separator;
			// 
			// tbbAbout
			// 
			this.tbbAbout.ImageIndex = 8;
			this.tbbAbout.ToolTipText = "About";
			// 
			// ilToolBar
			// 
			this.ilToolBar.ImageSize = new System.Drawing.Size(16, 16);
			this.ilToolBar.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("ilToolBar.ImageStream")));
			this.ilToolBar.TransparentColor = System.Drawing.Color.Transparent;
			// 
			// splitter1
			// 
			this.splitter1.Dock = System.Windows.Forms.DockStyle.Top;
			this.splitter1.Location = new System.Drawing.Point(0, 24);
			this.splitter1.Name = "splitter1";
			this.splitter1.Size = new System.Drawing.Size(528, 3);
			this.splitter1.TabIndex = 4;
			this.splitter1.TabStop = false;
			// 
			// tvResults
			// 
			this.tvResults.Dock = System.Windows.Forms.DockStyle.Fill;
			this.tvResults.ImageList = this.ilTree;
			this.tvResults.Location = new System.Drawing.Point(0, 27);
			this.tvResults.Name = "tvResults";
			this.tvResults.Size = new System.Drawing.Size(528, 301);
			this.tvResults.TabIndex = 5;
			this.tvResults.DoubleClick += new System.EventHandler(this.tvResults_DoubleClick);
			this.tvResults.AfterSelect += new System.Windows.Forms.TreeViewEventHandler(this.tvResults_AfterSelect);
			this.tvResults.BeforeCollapse += new System.Windows.Forms.TreeViewCancelEventHandler(this.tvResults_BeforeCollapse);
			this.tvResults.BeforeExpand += new System.Windows.Forms.TreeViewCancelEventHandler(this.tvResults_BeforeExpand);
			// 
			// saveFileDialog1
			// 
			this.saveFileDialog1.FileName = "doc1";
			// 
			// vsnunitctl
			// 
			this.Controls.Add(this.tvResults);
			this.Controls.Add(this.splitter1);
			this.Controls.Add(this.panel1);
			this.Name = "vsnunitctl";
			this.Size = new System.Drawing.Size(528, 328);
			this.Load += new System.EventHandler(this.vsnunitctl_Load);
			this.panel1.ResumeLayout(false);
			this.ResumeLayout(false);

		}

		#endregion

		private void OnAfterProjectClosd()
		{
			tvResults.Nodes.Clear();
			
		}

		private void openFileDialog1_FileOk(object sender, System.ComponentModel.CancelEventArgs e)
		{
		
		}
	}



	


/*
	#region UnitTests



	[TestFixture]

	public class TestVSNUnit2003

	{

		private vsnunitctl vsn;

		private const string gooddll = @"c:\temp\temptest\bin\debug\temptest.exe";

		private const string baddll = @"c:\temp\temptestnonunit\bin\debug\temptestnonunit.exe";

		private const string stackline = "   at temptest.CTest.Test() in c:\temp\temptest\form1.cs:line 18";





		[SetUp]

		public void setup()

		{

			vsn = new vsnunitctl();

			vsn.Show();

			vsn.cnu.NUnitExec = @"c:\program files\NUnit v2.1\bin\nunit-console.exe";

			

		}



		[TearDown]

		public void teardown()

		{

			

			vsn = null;

		}



		[Test]

		public void TestBuild()

		{

			XmlDocument xd = vsn.TestAndBuild(gooddll);

			Assert.IsNotNull(xd);

			vsn.FillTreeview(xd);

			Assert.IsTrue(vsn.tvResults.Nodes.Count > 0);

		}



		[Test]

		public void TestNoTests()

		{

			XmlDocument xd = vsn.TestAndBuild(baddll);

			Assert.IsNotNull(xd);

			Assert.IsTrue(xd.SelectNodes("//message").Count == 0);

			vsn.FillTreeview(xd);

			Assert.IsTrue(vsn.tvResults.Nodes.Count == 1);

		}		



		[Test]

		public void TestParseStackLine()

		{

			TestCase tc = new TestCase();

			vsn.ParseSource(stackline, ref tc);

			Assert.IsTrue(tc.FileName == "c:\temp\temptest\form1.cs", "FileName is wrong: " + tc.FileName);

			Assert.IsTrue(tc.LineNumber == 18, "LineNumber is wrong: " + tc.LineNumber.ToString());

			

		}





	}



	#endregion
*/
}

